from ._z import Z
from plotly.graph_objs.surface.contours import z
from ._y import Y
from plotly.graph_objs.surface.contours import y
from ._x import X
from plotly.graph_objs.surface.contours import x
